from datetime import datetime
from threading import Thread
import logging
logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',level=logging.NOTSET)

class StageTrading(Thread):

      def __init__(self, exchange, telegram, symbol='ETH/USDT', step=1, count=3, factor=2):
          super().__init__()
          self.symbol=symbol
          self.step=step
          self.count=count
          self.factor=factor
          self.exchange=exchange
          self.telegram=telegram
          self.start=datetime.now()
          self.logger=logging.getLogger('StageStrategy')
          self.status='STOPPED'

      def Run(self):
            if (self.telegram.status=='START') & (self.status=='STOPPED'):
                  self.Start()
            if (self.telegram.status=='STOP') & (self.status=='RUN'):
                  self.Stop()
            if self.telegram.status=='RUN':
                  self.Update()
            
      def Start(self): ##start Method
            bid=self.exchange.Ticker(self.symbol)['bid']
            self.telegram.Message('starting at %f' % bid)
            self.logger.info('starting at %f' % bid)
            self.exchange.CancelAll(self.symbol)
            for i in range(0,self.count,1):
                  #Buy
                  price = round( bid*(1-self.step/100)**(i+1), self.exchange.markets[self.symbol]['precision']['price'])
                  order=self.Buy( price)
                  #Sell
                  price=bid*(1+self.step/100)**(i+1)
                  self.Sell(price)
                        
            self.logger.info('Stage Trading Started...')
            self.status='RUN'

      def Stop(self):
            self.exchange.CancelAll(self.symbol)
            self.telegram.Message('%s stopped' % self.symbol)
            self.status='STOPPED'
            self.logger.info('Stage Trading Stopped')

      def Amount(self,price):
            '''
            Get the volume in base currency using the defined budget and volume percentge
            '''
            minCost=self.exchange.MinCost(self.symbol)*self.factor
            return round(minCost/price,self.exchange.markets[self.symbol]['precision']['amount'])
            
      def Buy(self, price, amount=0):
            if amount==0:
                  amount=self.Amount(price)

            order=self.exchange.Buy(self.symbol, amount, price)
            if len(order)!=0:
                  self.telegram.Message("BUY Amount:%f Price:%f Cost:%f" %(order['amount'], order['price'], order['cost']))
                  self.logger.info("BUY Amount:%f Price:%f Cost:%f" %(order['amount'], order['price'], order['cost']))
                  return order
            return order

      def Sell(self, price, amount=0):
            if amount==0:
                  amount=self.Amount(price*(1-self.step/100)) #amount from buy position
                  amount=amount*(1-self.exchange.markets[self.symbol]['taker']/100) #amount corrected with fees
                  amount=round(amount,self.exchange.markets[self.symbol]['precision']['amount'])

            order=self.exchange.Sell(self.symbol, amount, price)
            if len(order)!=0:
                  self.telegram.Message("SELL Volume:%f Price:%f Cost:%f" %(order['amount'], order['price'], order['cost']))
                  self.logger.info("SELL Volume:%f Price:%f Cost:%f" %(order['amount'], order['price'], order['cost']))
                  return order
            return order

      def Update(self): ##update the positions
            #self.Up()
            self.FilledBuy()
            self.BuyMax()
            self.FilledSell()
            self.SellMax()

      def Up(self):
            symbol=self.symbol
            bid=self.exchange.Ticker(symbol)['bid']
            if (self.exchange.Highest(symbol,'buy')['price']*(1+self.step*1.5/100)>bid):
                  return
            self.logger.debug('Price moving up')

            if self.exchange.Highest(symbol,'buy')['price']*(1+self.step*1.5/100)<bid:
                  highest=self.exchange.Highest(symbol,'buy')
                  price=highest['price']*(1+self.step/100)
                  self.logger.debug('Buy position to be opened at %f' % price)
                  if len(self.Buy(price))==0:
                        self.logger.debug('Open buy position failed, exiting UpdateAboveBuy')
            self.BuyMax()
            self.logger.debug('Price moving up')

      def FilledBuy(self):
            symbol=self.symbol
            bid=self.exchange.Ticker(symbol)['bid']
            if self.exchange.Count(symbol,'buy') >= self.count:
                  return

            filled=self.exchange.LastFilled(symbol)
            if filled['side']!='buy':
                  return

            self.logger.info('Position has been filled')
            #open sell
            amount=filled['amount']
            self.telegram.Message("BUY Position closed Amount:%f Price:%f Cost:%f" %(filled['amount'], filled['price'], filled['cost']))
            self.logger.info("BUY Position closed Amount:%f Price:%f Cost:%f" %(filled['amount'], filled['price'], filled['cost']))
            self.Sell(filled['price']*(1+self.step/100),amount)
            
            #open buy below
            lowest=self.exchange.Lowest(symbol,'buy')
            price=lowest['price']*(1-self.step/100)
            if len(lowest)==0:
                   self.logger.info('Lowest is empty !!!!')
                   return
            if len(self.Buy(price))==0:
                   self.logger.info('Buy position could not be created')
            self.logger.info('Buy filled completed')

      def FilledSell(self):
            symbol=self.symbol
            bid=self.exchange.Ticker(symbol)['bid']
            if self.exchange.Count(symbol,'sell') >= self.count:
                  return

            filled=self.exchange.LastFilled(symbol)
            if filled['side']!='sell':
                  return

            self.logger.info('Sell Position has been filled')
            #open sell
            amount=filled['amount']
            self.telegram.Message("SELL Position closed Amount:%f Price:%f Cost:%f" %(filled['amount'], filled['price'], filled['cost']))
            self.logger.info("SELL Position closed Amount:%f Price:%f Cost:%f" %(filled['amount'], filled['price'], filled['cost']))
            
            self.Buy(filled['price']*(1-self.step/100),amount)
            
            #open buy below
            highest=self.exchange.Highest(symbol,'sell')
            amount=self.Amount(highest['price'])
            price=highest['price']*(1+self.step/100)
            if len(self.Sell(price,amount))==0:
                   self.logger.info('Sell position could not be created')
            self.logger.info('Sell filled completed')
            
      def BuyMax(self):
            symbol=self.symbol
            if self.exchange.Count(symbol,'buy') <= self.count:
                  return

            self.logger.debug('%s: Max buy count exceeded' %symbol)
            lowest=self.exchange.Lowest(symbol,'buy')
            res=self.exchange.Cancel(symbol,lowest['id'])
#            if len(self.exchange.Cancel(symbol,lowest['id']))!=0:
#                  self.telegram.Message("BUY Position Price:%f Amounnt:%f Cancelled" %(lowest['price'],lowest['amount']))
            
      def SellMax(self):
            symbol=self.symbol

            if self.exchange.Count(symbol,'sell') <= self.count:
                  return

            self.logger.debug('Max sell count exceeded')
            lowest=self.exchange.Lowest(symbol,'sell')
            res=self.exchange.Cancel(symbol,lowest['id'])
#            if len(self.exchange.Cancel(symbol,lowest['id']))!=0:
#                  self.telegram.Message("Sell Position ID:%s Cancelled" %lowest['id'])
